@@ -16,7 +16,8 @@ from psyplot_gui.config.rcsetup import rcParams
 from psyplot_gui.compat.qtcompat import (
     QWidget, QHBoxLayout, QFrame, QVBoxLayout, QWebEngineView, QToolButton,
     QIcon, QtCore, QComboBox, Qt,  QSortFilterProxyModel, isstring, asstring,
-    QCompleter, QStandardItemModel, QPlainTextEdit, QAction, QMenu, with_qt5)
+    QCompleter, QStandardItemModel, QPlainTextEdit, QAction, QMenu, with_qt5,
+    QtGui)
 from psyplot_gui.common import get_icon, DockMixin, PyErrorMessage
 from IPython.core.oinspect import signature, getdoc
 import logging
@@ -148,23 +149,47 @@ class UrlBrowser(QFrame):
     #: displayed
     default_url = None
 
+    #: adress line
+    tb_url = None
+
+    #: button to go to previous url
+    bt_back = None
+
+    #: button to go to next url
+    bt_ahead = None
+
+    #: refresh the current url
+    bt_refresh = None
+
+    #: button to go lock to the current url
+    bt_lock = None
+
+    #: button to disable browsing in www
+    bt_url_lock = None
+
+    #: The upper part of the browser containing all the buttons
+    button_box = None
+
+    #: The upper most layout aranging the button box and the html widget
+    vbox = None
+
     def __init__(self, *args, **kwargs):
         super(UrlBrowser, self).__init__(*args, **kwargs)
 
         # ---------------------------------------------------------------------
         # ---------------------------- upper buttons --------------------------
         # ---------------------------------------------------------------------
-        #: adress line
+        # adress line
         self.tb_url = UrlCombo(self)
-        #: button to go to previous url
+        # button to go to previous url
         self.bt_back = QToolButton(self)
-        #: button to go to next url
+        # button to go to next url
         self.bt_ahead = QToolButton(self)
-        #: refresh the current url
+        # refresh the current url
         self.bt_refresh = QToolButton(self)
-        #: button to go lock to the current url
+        # button to go lock to the current url
         self.bt_lock = QToolButton(self)
-        #: button to disable browsing in www
+        # button to disable browsing in www
         self.bt_url_lock = QToolButton(self)
 
         # ---------------------------- buttons settings -----------------------
@@ -217,7 +242,7 @@ class UrlBrowser(QFrame):
         # ---------------------------- layouts --------------------------------
         # ---------------------------------------------------------------------
 
-        #: The upper part of the browser containing all the buttons
+        # The upper part of the browser containing all the buttons
         self.button_box = button_box = QHBoxLayout()
 
         button_box.addWidget(self.bt_back)
@@ -227,7 +252,7 @@ class UrlBrowser(QFrame):
         button_box.addWidget(self.bt_lock)
         button_box.addWidget(self.bt_url_lock)
 
-        #: The upper most layout aranging the button box and the html widget
+        # The upper most layout aranging the button box and the html widget
         self.vbox = vbox = QVBoxLayout()
         self.vbox.setContentsMargins(0, 0, 0, 0)
         vbox.addLayout(button_box)
@@ -267,7 +292,7 @@ class UrlBrowser(QFrame):
         logger.debug('url changed to %s', url)
         try:
             self.tb_url.setCurrentText(url)
-        except AttributeError:
+        except AttributeError:  # Qt4
             self.tb_url.setEditText(url)
         self.tb_url.add_text_on_top(url, block=True)
 
@@ -314,7 +339,7 @@ class HelpMixin(object):
 
     @docstrings.get_sectionsf('HelpMixin.show_help')
     @docstrings.dedent
-    def show_help(self, obj, oname=''):
+    def show_help(self, obj, oname='', files=None):
         """
         Show the rst documentation for the given object
 
@@ -323,10 +348,13 @@ class HelpMixin(object):
         obj: object
             The object to get the documentation for
         oname: str
-            The name to use for the object in the documentation"""
+            The name to use for the object in the documentation
+        files: list of str
+            A path to additional files that shall be used to process the
+            docs"""
         descriptor = self.describe_object(obj, oname)
         doc = self.get_doc(descriptor)
-        self.show_rst(doc, descriptor=descriptor)
+        return self.show_rst(doc, descriptor=descriptor, files=files)
 
     def header(self, descriptor, sig):
         """Format the header and include object name and signature `sig`
@@ -412,7 +440,7 @@ class HelpMixin(object):
 
     @docstrings.get_sectionsf('HelpMixin.show_rst')
     @docstrings.dedent
-    def show_rst(self, text, oname='', descriptor=None):
+    def show_rst(self, text, oname='', descriptor=None, files=None):
         """
         Abstract method which needs to be implemented by th widget to show
         restructured text
@@ -424,8 +452,16 @@ class HelpMixin(object):
         oname: str
             The object name
         descriptor: instance of :attr:`object_descriptor`
-            The object descriptor holding the informations"""
-        pass
+            The object descriptor holding the informations
+        files: list of str
+            A path to additional files that shall be used to display the docs
+
+        Returns
+        -------
+        bool
+            True if the text is displayed
+        """
+        return False
 
     @docstrings.get_sectionsf('HelpMixin.show_intro')
     def show_intro(self, text=''):
@@ -451,6 +487,7 @@ class TextHelp(QFrame, HelpMixin):
         #: The :class:`PyQt5.QtWidgets.QPlainTextEdit` instance used for
         #: displaying the documentation
         self.editor = QPlainTextEdit(parent=self)
+        self.editor.setFont(QtGui.QFont('Courier New'))
         self.vbox.addWidget(self.editor)
         self.setLayout(self.vbox)
 
@@ -465,6 +502,7 @@ class TextHelp(QFrame, HelpMixin):
             Are ignored"""
         self.editor.clear()
         self.editor.insertPlainText(text)
+        return True
 
 
 class UrlHelp(UrlBrowser, HelpMixin):
@@ -479,8 +517,15 @@ class UrlHelp(UrlBrowser, HelpMixin):
     can_document_object = with_sphinx
     can_show_rst = with_sphinx
 
+    #: menu button with different urls
+    bt_url_menus = None
+
+    #:
+    sphinx_thread = None
+
     def __init__(self, *args, **kwargs):
-        self.sphinx_dir = kwargs.pop('sphinx_dir', mkdtemp())
+        self._temp_dir = 'sphinx_dir' not in kwargs
+        self.sphinx_dir = kwargs.pop('sphinx_dir', mkdtemp(prefix='psyplot_'))
         self.build_dir = osp.join(self.sphinx_dir, '_build', 'html')
         super(UrlHelp, self).__init__(*args, **kwargs)
 
@@ -498,9 +543,6 @@ class UrlHelp(UrlBrowser, HelpMixin):
             rcParams.connect('help_explorer.online',
                              self.reset_sphinx)
 
-        else:
-            self.sphinx_thread = Nonec
-
         self.bt_connect_console = QToolButton(self)
         self.bt_connect_console.setCheckable(True)
         if rcParams['console.connect_to_help']:
@@ -515,7 +557,7 @@ class UrlHelp(UrlBrowser, HelpMixin):
                          self.update_connect_console)
         self.toogle_connect_console()
 
-        #: menu button with different urls
+        # menu button with different urls
         self.bt_url_menus = QToolButton(self)
         self.bt_url_menus.setIcon(QIcon(get_icon('docu_button.png')))
         self.bt_url_menus.setToolTip('Browse documentations')
@@ -557,7 +599,7 @@ class UrlHelp(UrlBrowser, HelpMixin):
             del self.sphinx_thread.app
 
     @docstrings.dedent
-    def show_help(self, obj, oname=''):
+    def show_help(self, obj, oname='', files=None):
         """
         Render the rst docu for the given object with sphinx and show it
 
@@ -567,7 +609,7 @@ class UrlHelp(UrlBrowser, HelpMixin):
         """
         if self.bt_lock.isChecked():
             return
-        return super(UrlHelp, self).show_help(obj, oname=oname)
+        return super(UrlHelp, self).show_help(obj, oname=oname, files=files)
 
     @docstrings.dedent
     def show_intro(self, text=''):
@@ -579,20 +621,25 @@ class UrlHelp(UrlBrowser, HelpMixin):
         %(HelpMixin.show_intro.parameters)s"""
         if self.sphinx_thread is not None:
             with open(self.sphinx_thread.index_file, 'a') as f:
-                f.write(text)
+                f.write('\n' + text.strip() + '\n\n' +
+                        'Table of Contents\n'
+                        '=================\n\n.. toctree::\n')
             self.sphinx_thread.render(None, None)
 
-    def show_rst(self, text, oname='', descriptor=None):
+    def show_rst(self, text, oname='', descriptor=None, files=None):
         """Render restructured text with sphinx and show it
 
         Parameters
         ----------
         %(HelpMixin.show_rst.parameters)s"""
-        if self.bt_lock.isChecked():
-            return
+        if self.bt_lock.isChecked() or self.sphinx_thread is None:
+            return False
         if not oname and descriptor:
             oname = descriptor.name
+        for f in files or []:
+            shutil.copyfile(f, osp.join(self.sphinx_dir, osp.basename(f)))
         self.sphinx_thread.render(text, oname)
+        return True
 
     def describe_object(self, obj, oname=''):
         """Describe an object using additionaly the object type from the
@@ -679,7 +726,7 @@ class UrlHelp(UrlBrowser, HelpMixin):
         try:
             get_module_source(modname)
             return True
-        except:
+        except Exception:
             return False
 
     def get_doc(self, descriptor):
@@ -751,6 +798,18 @@ class UrlHelp(UrlBrowser, HelpMixin):
         return indent(super(UrlHelp, self).process_docstring(
             lines, descriptor))
 
+    def close(self, *args, **kwargs):
+        if self.sphinx_thread is not None:
+            try:
+                del self.sphinx_thread.app
+            except AttributeError:
+                pass
+            shutil.rmtree(self.build_dir, ignore_errors=True)
+            if self._temp_dir:
+                shutil.rmtree(self.sphinx_dir, ignore_errors=True)
+            del self.sphinx_thread
+        return super(UrlHelp, self).close(*args, **kwargs)
+
 
 class SphinxThread(QtCore.QThread):
     """A thread to render sphinx documentation in a separate process"""
@@ -826,7 +885,7 @@ class SphinxThread(QtCore.QThread):
                 self.outdir, '_build', 'html', 'psyplot.html'))
         try:
             self.app.build(None, [])
-        except:
+        except Exception:
             msg = 'Error while building sphinx document %s' % (
                 self.name)
             self.html_error.emit('<b>' + msg + '</b>')
@@ -892,7 +951,7 @@ class HelpExplorer(QWidget, DockMixin):
             self.combo.setCurrentIndex(list(self.viewers).index(name))
 
     @docstrings.dedent
-    def show_help(self, obj, oname=''):
+    def show_help(self, obj, oname='', files=None):
         """
         Show the documentaion of the given object
 
@@ -904,27 +963,33 @@ class HelpExplorer(QWidget, DockMixin):
         ----------
         %(HelpMixin.show_help.parameters)s"""
         oname = asstring(oname)
+        ret = None
         if self.viewer.can_document_object:
             try:
-                return self.viewer.show_help(obj, oname=oname)
-            except:
+                ret = self.viewer.show_help(obj, oname=oname, files=files)
+            except Exception:
                 logger.debug("Could not document %s with %s viewer!",
                              oname, self.combo.currentText(), exc_info=True)
-        curr_i = self.combo.currentIndex()
-        for i, (viewername, viewer) in enumerate(six.iteritems(self.viewers)):
-            if i != curr_i and viewer.can_document_object:
-                self.set_viewer(viewername)
-                self.combo.blockSignals(True)
-                self.combo.setCurrentIndex(i)
-                self.combo.blockSignals(False)
-                try:
-                    return viewer.show_help(obj, oname=oname)
-                except:
-                    logger.debug("Could not document %s with %s viewer!",
-                                 oname, viewername, exc_info=True)
+        else:
+            curr_i = self.combo.currentIndex()
+            for i, (viewername, viewer) in enumerate(
+                    six.iteritems(self.viewers)):
+                if i != curr_i and viewer.can_document_object:
+                    self.set_viewer(viewername)
+                    self.combo.blockSignals(True)
+                    self.combo.setCurrentIndex(i)
+                    self.combo.blockSignals(False)
+                    try:
+                        ret = viewer.show_help(obj, oname=oname, files=files)
+                    except Exception:
+                        logger.debug("Could not document %s with %s viewer!",
+                                     oname, viewername, exc_info=True)
+        if ret:
+            self.dock.raise_()
+        return ret
 
     @docstrings.dedent
-    def show_rst(self, text, oname=''):
+    def show_rst(self, text, oname='', files=None):
         """
         Show restructured text
 
@@ -935,12 +1000,18 @@ class HelpExplorer(QWidget, DockMixin):
         Parameters
         ----------
         %(HelpMixin.show_rst.parameters)s"""
+        ret = None
         if self.viewer.can_show_rst:
-            return self.viewer.show_rst(text, oname=oname)
-        for viewer in six.itervalues(self.viewers):
-            if viewer.can_show_rst:
-                self.set_viewer(viewer)
-                return viewer.show_rst(text, oname=oname)
+            ret = self.viewer.show_rst(text, oname=oname, files=files)
+        else:
+            for viewer in six.itervalues(self.viewers):
+                if viewer.can_show_rst:
+                    self.set_viewer(viewer)
+                    ret = viewer.show_rst(text, oname=oname, files=files)
+                    break
+        if ret:
+            self.dock.raise_()
+        return ret
 
     @docstrings.dedent
     def show_intro(self, text=''):
@@ -961,3 +1032,7 @@ class HelpExplorer(QWidget, DockMixin):
                 if i:
                     self.set_viewer(viewer)
                 found = True
+
+    def close(self, *args, **kwargs):
+        self.viewers['HTML help'].close(*args, **kwargs)
+        return super(HelpExplorer, self).close(*args, **kwargs)
